package cn.ibizlab.core.extensions.service;

import cn.ibizlab.core.lite.extensions.service.DbEntityService;
import cn.ibizlab.core.rule.service.impl.ExecResultServiceImpl;
import lombok.extern.slf4j.Slf4j;
import cn.ibizlab.core.rule.domain.ExecResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Primary;
import java.util.*;

/**
 * 实体[执行结果] 自定义服务对象
 */
@Slf4j
@Primary
@Service("ExecResultExService")
public class ExecResultExService extends ExecResultServiceImpl {

    @Autowired
    private DbEntityService dbEntityService;

    private ArrayList<ExecResult> resultsSaveAsync=new ArrayList<ExecResult>();
    private Object objResultSaveAsyncLock=new Object();
    private Timer timerSaveAsync=null;

    private ArrayList<ExecResult> resultsDeleteAsync=new ArrayList<ExecResult>();
    private Object objResultDeleteAsyncLock=new Object();
    private Timer timerDeleteAsync=null;

    @Override
    protected Class currentModelClass() {
        return com.baomidou.mybatisplus.core.toolkit.ReflectionKit.getSuperClassGenericType(this.getClass().getSuperclass(), 1);
    }

    /**
     * [SaveAsync:SaveAsync] 行为扩展
     * @param et
     * @return
     */
    @Override
    @Transactional
    public ExecResult saveAsync(ExecResult et) {
        synchronized(objResultSaveAsyncLock)
        {
            this.resultsSaveAsync.add(et);
        }
        //创建一个定时器
        if(timerSaveAsync==null)
        {
            timerSaveAsync = new Timer();
            //schedule方法是执行时间定时任务的方法
            timerSaveAsync.schedule(new TimerTask() {

                //run方法就是具体需要定时执行的任务
                @Override
                public void run() {
                    try
                    {

                        synchronized(objResultSaveAsyncLock)
                        {
                            if(resultsSaveAsync.size()>0)
                            {
                                List<ExecResult> args=new ArrayList<>();

                                for(ExecResult arg:resultsSaveAsync)
                                {
                                    args.add(arg);
                                    if(args.size()>=500)
                                    {
//                                        TreatEarlyWarningThread task = new TreatEarlyWarningThread(args);
//                                        task.start();
                                        dbEntityService.saveResultBatch(args);
                                        args.clear();
                                    }

                                }
                                if(args.size()>0)
                                {
//                                    TreatEarlyWarningThread task = new TreatEarlyWarningThread(args);
//                                    task.start();
                                    dbEntityService.saveResultBatch(args);
                                    args.clear();
                                }
                                resultsSaveAsync.clear();
                            }

                        }
                    }
                    catch (Exception ex)
                    {
                        log.error(ex.getMessage());
                    }

                }
            }, 1000, 10000);
        }
        return et;
    }

    /**
     * [DeleteAsync:DeleteAsync] 行为扩展
     * @param et
     * @return
     */
    @Override
    @Transactional
    public ExecResult deleteAsync(ExecResult et) {
        synchronized(objResultDeleteAsyncLock)
        {
            this.resultsDeleteAsync.add(et);
        }
        if(timerDeleteAsync==null)
        {
            timerDeleteAsync = new Timer();
            //schedule方法是执行时间定时任务的方法
            timerDeleteAsync.schedule(new TimerTask() {

                //run方法就是具体需要定时执行的任务
                @Override
                public void run() {
                    try
                    {

                        synchronized(objResultDeleteAsyncLock)
                        {
                            if(resultsDeleteAsync.size()>0)
                            {
                                List<ExecResult> args=new ArrayList<>();

                                for(ExecResult arg:resultsDeleteAsync)
                                {
                                    args.add(arg);
                                    if(args.size()>=500)
                                    {
//                                        UpdateTreatEarlyWarningThread updateTreatEarlyWarningThread = new UpdateTreatEarlyWarningThread(args);
//                                        updateTreatEarlyWarningThread.start();
                                        dbEntityService.clearResultBatch(args);
                                        args.clear();
                                    }
                                }

                                if(args.size()>=0)
                                {
//                                    UpdateTreatEarlyWarningThread updateTreatEarlyWarningThread = new UpdateTreatEarlyWarningThread(args);
//                                    updateTreatEarlyWarningThread.start();
                                    dbEntityService.clearResultBatch(args);
                                    args.clear();
                                }
                                resultsDeleteAsync.clear();
                            }

                        }
                    }
                    catch (Exception ex)
                    {
                        log.error(ex.getMessage());
                    }

                }
            }, 1000, 10000);
        }
        return et;
    }
}

